@@ -2,7 +2,7 @@
 #include "dp.h"
 
 #define MACHIGAI 0
-#define OUTGAP0TRY 1
+#define OUTGAP0TRY 0
 #define DEBUG 0
 #define XXXXXXX    0
 #define USE_PENALTY_EX  0
@@ -83,6 +83,18 @@ void imp_match_init_strict( float *imp, int clus1, int clus2, int lgth1, int lgt
 	static TLS char *nocount2 = NULL;
 	LocalHom *tmpptr;
 
+	if( seq1 == NULL )
+	{
+		if( impmtx ) FreeFloatMtx( impmtx );
+		impmtx = NULL;
+		if( nocount1 ) free( nocount1 );
+		nocount1 = NULL;
+		if( nocount2 ) free( nocount2 );
+		nocount2 = NULL;
+		
+		return;
+	}
+
 	if( impalloclen < lgth1 + 2 || impalloclen < lgth2 + 2 )
 	{
 		if( impmtx ) FreeFloatMtx( impmtx );
@@ -761,7 +773,8 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
 						char **seq1, char **seq2, 
                         char **mseq1, char **mseq2, 
                         float **cpmx1, float **cpmx2, 
-                        int **ijp, int icyc, int jcyc )
+                        int **ijp, int icyc, int jcyc,
+						int tailgp )
 {
 	int i, j, l, iin, jin, ifi, jfi, lgth1, lgth2, k;
 	float wm;
@@ -780,7 +793,7 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
 	}
 #endif
  
-	if( outgap == 1 )
+	if( tailgp == 1 )
 		;
 	else
 	{
@@ -864,7 +877,7 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
 	return( 0.0 );
 }
 
-float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc, int jcyc, int alloclen, LocalHom ***localhom, float *impmatch, char *sgap1, char *sgap2, char *egap1, char *egap2 )
+float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc, int jcyc, int alloclen, LocalHom ***localhom, float *impmatch, char *sgap1, char *sgap2, char *egap1, char *egap2, int *chudanpt, int chudanref, int *chudanres, int headgp, int tailgp )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	int k;
@@ -911,12 +924,59 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	float ogcp1va;
 
 
+	if( seq1 == NULL )
+	{
+		if( orlgth1 )
+		{
+//			fprintf( stderr, "## Freeing local arrays in A__align\n" );
+			orlgth1 = 0;
+			orlgth2 = 0;
+
+			imp_match_init_strict( NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0 );
+
+			free( mseq1 );
+			free( mseq2 );
+			FreeFloatVec( w1 );
+			FreeFloatVec( w2 );
+			FreeFloatVec( match );
+			FreeFloatVec( initverticalw );
+			FreeFloatVec( lastverticalw );
+
+			FreeFloatVec( m );
+			FreeIntVec( mp );
+
+			FreeCharMtx( mseq );
+
+			FreeFloatVec( ogcp1 );
+			FreeFloatVec( ogcp2 );
+			FreeFloatVec( fgcp1 );
+			FreeFloatVec( fgcp2 );
+
+
+			FreeFloatMtx( cpmx1 );
+			FreeFloatMtx( cpmx2 );
+
+			FreeFloatMtx( floatwork );
+			FreeIntMtx( intwork );
+
+		}
+		else
+		{
+//			fprintf( stderr, "## Not allocated\n" );
+		}
+		return( 0.0 );
+	}
+
+
 
 #if 0
 	fprintf( stderr, "####  eff in SA+++align\n" );
 	fprintf( stderr, "####  seq1[0] = %s\n", seq1[0] );
 	fprintf( stderr, "####  strlen( seq1[0] ) = %d\n", strlen( seq1[0] ) );
 	for( i=0; i<icyc; i++ ) fprintf( stderr, "eff1[%d] = %f\n", i, eff1[i] );
+	fprintf( stderr, "####  seq2[0] = %s\n", seq2[0] );
+	fprintf( stderr, "####  strlen( seq2[0] ) = %d\n", strlen( seq2[0] ) );
+	for( i=0; i<jcyc; i++ ) fprintf( stderr, "eff2[%d] = %f\n", i, eff2[i] );
 #endif
 	if( orlgth1 == 0 )
 	{
@@ -1105,7 +1165,7 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 
 #endif
 
-	if( outgap == 1 )
+	if( headgp == 1 )
 	{
 		for( i=1; i<lgth1+1; i++ )
 		{
@@ -1135,7 +1195,7 @@ float A__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc,
 	else
 		lastverticalw[0] = currentw[lgth2-1];
 
-	if( outgap ) lasti = lgth1+1; else lasti = lgth1;
+	if( tailgp ) lasti = lgth1+1; else lasti = lgth1;
 
 #if XXXXXXX
 fprintf( stderr, "currentw = \n" );
@@ -1159,6 +1219,16 @@ for( i=0; i<lgth2; i++ )
 
 	for( i=1; i<lasti; i++ )
 	{
+
+#ifdef enablemultithread
+//		fprintf( stderr, "chudan = %d, %d\n", *chudanpt, chudanref );
+		if( chudanpt && *chudanpt != chudanref ) 
+		{
+//			fprintf( stderr, "\n\n## CHUUDAN!!! S\n" );
+			*chudanres = 1;
+			return( -1.0 );
+		}
+#endif
 		wtmp = previousw; 
 		previousw = currentw;
 		currentw = wtmp;
@@ -1211,6 +1281,15 @@ fprintf( stderr, "\n" );
 		lastj = lgth2+1;
 		for( j=1; j<lastj; j++ )
 		{
+#ifdef xxxenablemultithread
+//			fprintf( stderr, "chudan = %d, %d\n", *chudanpt, chudanref );
+			if( chudanpt && *chudanpt != chudanref ) 
+			{
+//				fprintf( stderr, "\n\n## CHUUDAN!!! S\n" );
+				*chudanres = 1;
+				return( -1.0 );
+			}
+#endif
 			wm = *prept;
 			*ijppt = 0;
 
@@ -1291,7 +1370,7 @@ fprintf( stderr, "\n" );
 		Atracking_localhom( impmatch, currentw, lastverticalw, seq1, seq2, mseq1, mseq2, cpmx1, cpmx2, ijp, icyc, jcyc );
 	}
 	else
-		Atracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, cpmx1, cpmx2, ijp, icyc, jcyc );
+		Atracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, cpmx1, cpmx2, ijp, icyc, jcyc, tailgp );
 
 //	fprintf( stderr, "### impmatch = %f\n", *impmatch );
 
@@ -1305,12 +1384,12 @@ fprintf( stderr, "\n" );
 
 	for( i=0; i<icyc; i++ ) strcpy( seq1[i], mseq1[i] );
 	for( j=0; j<jcyc; j++ ) strcpy( seq2[j], mseq2[j] );
-	/*
+#if 0
 	fprintf( stderr, "\n" );
 	for( i=0; i<icyc; i++ ) fprintf( stderr, "%s\n", mseq1[i] );
 	fprintf( stderr, "#####\n" );
 	for( j=0; j<jcyc; j++ ) fprintf( stderr, "%s\n", mseq2[j] );
-	*/
+#endif
 
 //	fprintf( stderr, "wm = %f\n", wm );
 
@@ -1537,7 +1616,7 @@ float A__align_gapmap( char **seq1, char **seq2, double *eff1, double *eff2, int
 
 #endif
 
-	if( outgap == 1 )
+	if( 1 ) // tsuneni outgap=1
 	{
 		for( i=1; i<lgth1+1; i++ )
 		{
@@ -1568,7 +1647,7 @@ float A__align_gapmap( char **seq1, char **seq2, double *eff1, double *eff2, int
 	else
 		lastverticalw[0] = currentw[lgth2-1];
 
-	if( outgap ) lasti = lgth1+1; else lasti = lgth1;
+	if( 1 ) lasti = lgth1+1; else lasti = lgth1; // tsuneni outgap=1
 
 #if XXXXXXX
 fprintf( stderr, "currentw = \n" );
@@ -1727,7 +1806,7 @@ fprintf( stderr, "\n" );
 		Atracking_localhom_gapmap( impmatch, currentw, lastverticalw, seq1, seq2, mseq1, mseq2, cpmx1, cpmx2, ijp, icyc, jcyc, gapmap1, gapmap2 );
 	}
 	else
-		Atracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, cpmx1, cpmx2, ijp, icyc, jcyc );
+		Atracking( currentw, lastverticalw, seq1, seq2, mseq1, mseq2, cpmx1, cpmx2, ijp, icyc, jcyc, 1 );
 
 //	fprintf( stderr, "### impmatch = %f\n", *impmatch );
 
