@@ -210,9 +210,6 @@ void arguments( int argc, char *argv[] )
 				case 'e':
 					fftscore = 0;
 					break;
-				case 'O':
-					fftNoAnchStop = 1;
-					break;
 #if 0
 				case 'R':
 					fftRepeatStop = 1;
@@ -284,9 +281,15 @@ void arguments( int argc, char *argv[] )
 				case 'd':
 					disp = 1;
 					break;
-				case 'o':
+#if 1
+				case 'O':
 					outgap = 0;
 					break;
+#else
+				case 'O':
+					fftNoAnchStop = 1;
+					break;
+#endif
 				case 'J':
 					tbutree = 0;
 					break;
@@ -392,6 +395,13 @@ int commonsextet_p( short *table, int *pointt )
 	static TLS int *ct = NULL;
 	static TLS int *cp;
 
+	if( table == NULL )
+	{
+		if( memo ) free( memo );
+		if( ct ) free( ct );
+		return( 0 );
+	}
+
 	if( *pointt == -1 )
 		return( 0 );
 
@@ -500,6 +510,7 @@ static void *distancematrixthread( void *arg )
 		if( i == njob )
 		{
 			pthread_mutex_unlock( targ->mutex );
+			commonsextet_p( NULL, NULL );
 			return( NULL );
 		}
 		*jobpospt = i+1;
@@ -583,6 +594,16 @@ static void *treebasethread( void *arg )
 		if( l == njob-1 )
 		{
 			pthread_mutex_unlock( targ->mutex );
+			if( commonIP ) FreeIntMtx( commonIP );
+			commonIP = NULL;
+			Falign( NULL, NULL, NULL, NULL, 0, 0, 0, NULL, NULL, 0, NULL );
+			free( mseq1 );
+			free( mseq2 );
+			free( localcopy );
+			free( effarr1 );
+			free( effarr2 );
+			free( indication1 );
+			free( indication2 );
 			return( NULL );
 		}
 		*jobpospt = l+1;
@@ -708,7 +729,7 @@ static void *treebasethread( void *arg )
 */
 
 
-		if( !nevermemsave && ( alg != 'M' && ( len1 > 10000 || len2 > 10000  ) ) )
+		if( !nevermemsave && ( alg != 'M' && ( len1 > 30000 || len2 > 30000  ) ) )
 		{
 			fprintf( stderr, "\nlen1=%d, len2=%d, Switching to the memsave mode\n", len1, len2 );
 			alg = 'M';
@@ -729,13 +750,11 @@ static void *treebasethread( void *arg )
 			if( alg == 'M' )
 			{
 				fprintf( stderr, "m" );
-//				fprintf( stderr, "%d-%d", clus1, clus2 );
 				pscore = Falign_udpari_long( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, fftlog+m1 );
 			}
 			else
 			{
-//				fprintf( stderr, "%d-%d", clus1, clus2 );
-				pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, fftlog+m1 );
+				pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, fftlog+m1, NULL, 0, NULL );
 			}
 		}
 		else
@@ -750,13 +769,13 @@ static void *treebasethread( void *arg )
 				case( 'M' ):
 					fprintf( stderr, "m" );
 //					fprintf( stderr, "%d-%d", clus1, clus2 );
-					pscore = MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, NULL, NULL, NULL );
+					pscore = MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 					break;
 				case( 'Q' ):
 					if( clus1 == 1 && clus2 == 1 && 0 )
 					{
 //							fprintf( stderr, "%d-%d", clus1, clus2 );
-							pscore = G__align11( mseq1, mseq2, *alloclen );
+							pscore = G__align11( mseq1, mseq2, *alloclen, outgap, outgap );
 					}
 					else
 					{
@@ -773,12 +792,12 @@ static void *treebasethread( void *arg )
 					if( clus1 == 1 && clus2 == 1 )
 					{
 //						fprintf( stderr, "%d-%d", clus1, clus2 );
-						pscore = G__align11( mseq1, mseq2, *alloclen );
+						pscore = G__align11( mseq1, mseq2, *alloclen, outgap, outgap );
 					}
 					else
 					{
 //						fprintf( stderr, "%d-%d", clus1, clus2 );
-						pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL );
+						pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 					}
 					break;
 				default:
@@ -1012,7 +1031,7 @@ static void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 */
 
 
-		if( !nevermemsave && ( alg != 'M' && ( len1 > 10000 || len2 > 10000  ) ) )
+		if( !nevermemsave && ( alg != 'M' && ( len1 > 30000 || len2 > 30000  ) ) )
 		{
 			fprintf( stderr, "\nlen1=%d, len2=%d, Switching to the memsave mode\n", len1, len2 );
 			alg = 'M';
@@ -1033,13 +1052,11 @@ static void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 			if( alg == 'M' )
 			{
 				fprintf( stderr, "m" );
-//				fprintf( stderr, "%d-%d", clus1, clus2 );
 				pscore = Falign_udpari_long( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, fftlog+m1 );
 			}
 			else
 			{
-//				fprintf( stderr, "%d-%d", clus1, clus2 );
-				pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, fftlog+m1 );
+				pscore = Falign( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, fftlog+m1, NULL, 0, NULL );
 			}
 		}
 		else
@@ -1054,13 +1071,13 @@ static void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 				case( 'M' ):
 					fprintf( stderr, "m" );
 //					fprintf( stderr, "%d-%d", clus1, clus2 );
-					pscore = MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, NULL, NULL, NULL );
+					pscore = MSalignmm( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 					break;
 				case( 'Q' ):
 					if( clus1 == 1 && clus2 == 1 && 0 )
 					{
 //							fprintf( stderr, "%d-%d", clus1, clus2 );
-							pscore = G__align11( mseq1, mseq2, *alloclen );
+							pscore = G__align11( mseq1, mseq2, *alloclen, outgap, outgap );
 					}
 					else
 					{
@@ -1077,12 +1094,12 @@ static void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 					if( clus1 == 1 && clus2 == 1 )
 					{
 //						fprintf( stderr, "%d-%d", clus1, clus2 );
-						pscore = G__align11( mseq1, mseq2, *alloclen );
+						pscore = G__align11( mseq1, mseq2, *alloclen, outgap, outgap );
 					}
 					else
 					{
 //						fprintf( stderr, "%d-%d", clus1, clus2 );
-						pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL );
+						pscore = A__align( mseq1, mseq2, effarr1, effarr2, clus1, clus2, *alloclen, NULL, &dumfl, NULL, NULL, NULL, NULL, NULL, 0, NULL, outgap, outgap );
 					}
 					break;
 				default:
@@ -1105,7 +1122,7 @@ static void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 			adjustgapmap( strlen( mseq2[0] )-len2nocommongap+len2, gapmap, mseq2[0] );
 			restorecommongaps( njob, aseq, topol[l][0], topol[l][1], gapmap, *alloclen );
 			findnewgaps( clus2, mseq2, gaplen );
-			insertnewgaps( njob, alreadyaligned, aseq, topol[l][0], topol[l][1], gaplen, *alloclen );
+			insertnewgaps( njob, alreadyaligned, aseq, topol[l][1], topol[l][0], gaplen, gapmap, *alloclen, alg );
 			for( i=0; i<njob; i++ ) eq2dash( aseq[i] );
 			for( i=0; (m=topol[l][0][i])>-1; i++ ) alreadyaligned[m] = 1;
 		}
@@ -1118,7 +1135,7 @@ static void treebase( int *nlen, char **aseq, int nadd, char *mergeoralign, char
 //			fprintf( stderr, "STEP2 mseq1[0] = %s\n", mseq1[0] );
 //			fprintf( stderr, "STEP2 mseq2[0] = %s\n", mseq2[0] );
 			findnewgaps( clus1, mseq1, gaplen );
-			insertnewgaps( njob, alreadyaligned, aseq, topol[l][0], topol[l][1], gaplen, *alloclen );
+			insertnewgaps( njob, alreadyaligned, aseq, topol[l][0], topol[l][1], gaplen, gapmap, *alloclen, alg );
 //			fprintf( stderr, "STEP3 mseq1[0] = %s\n", mseq1[0] );
 //			fprintf( stderr, "STEP3 mseq2[0] = %s\n", mseq2[0] );
 			for( i=0; i<njob; i++ ) eq2dash( aseq[i] );
@@ -1392,6 +1409,8 @@ int main( int argc, char *argv[] )
 				pthread_join( handle[i], NULL );
 			}
 			pthread_mutex_destroy( &mutex );
+			free( handle );
+			free( targ );
 		}
 		else
 #endif
@@ -1739,6 +1758,9 @@ int main( int argc, char *argv[] )
 		}
 		pthread_mutex_destroy( &mutex );
 		pthread_cond_destroy( &treecond );
+		free( handle );
+		free( targ );
+		free( fftlog );
 	}
 	else
 #endif
@@ -1774,3 +1796,4 @@ int main( int argc, char *argv[] )
 	SHOWVERSION;
 	return( 0 );
 }
+
