@@ -204,6 +204,16 @@ int alignableReagion( int    clus1, int    clus2,
 	fprintf( stderr, "eff2[0] = %f\n", eff2[0] );
 #endif
 
+	if( clus1 == 0 )
+	{
+		FreeDoubleVec( stra ); stra = NULL;
+		FreeDoubleVec( prf1 ); prf1 = NULL;
+		FreeDoubleVec( prf2 ); prf2 = NULL;
+		FreeIntVec( hat1 ); hat1 = NULL;
+		FreeIntVec( hat2 ); hat2 = NULL;
+		return( 0 );
+	}
+
 	if( prf1 == NULL )
 	{
 		prf1 = AllocateDoubleVec( 26 );
@@ -386,6 +396,20 @@ void blockAlign2( int *cut1, int *cut2, Segment **seg1, Segment **seg2, double *
 	static TLS double maxj, maxi;
 	static TLS int pointj, pointi;
 
+	if( cut1 == NULL) 
+	{
+		if( result1 )
+		{
+			free( result1 );
+			free( result2 );
+			free( ocut1 );
+			free( ocut2 );
+			FreeIntMtx( track );
+        	FreeDoubleMtx( crossscore );
+		}
+		return;
+	}
+
 	if( result1 == NULL )
 	{
 		result1 = AllocateIntVec( MAXSEG );
