@@ -36,6 +36,7 @@ extern void treeconstruction( char **seq, int nseq, int ***topol, double **len,
 extern float bscore_calc( char **seq, int s, double **eff );
 extern void AllocateTmpSeqs( char ***mseq2pt, char **mseq1pt, int locnlenmax );
 extern void FreeTmpSeqs( char **mseq2, char *mseq1 );
+extern void gappick_samestring( char *aseq );
 extern void gappick0( char *aseq, char *seq );
 extern void gappick( int nseq, int s, char **aseq, char **mseq2, 
 					 double **eff, double *effarr );
@@ -189,11 +190,13 @@ extern void FRead( FILE *fp, char name[][B], int nlen[], char **seq );
 extern void kake2hiku( char *str );
 extern void readDataforgaln( FILE *fp, char **name, int *nlen, char **seq );
 extern void readData( FILE *fp, char name[][B], int nlen[], char **seq );
+extern void readData_pointer_casepreserve( FILE *fp, char **name, int *nlen, char **seq );
 extern void readData_pointer( FILE *fp, char **name, int *nlen, char **seq );
 extern void readData_pointer2( FILE *fp, int nseq, char **name, int *nlen, char **seq );
 extern void readData_varlen( FILE *fp, char **name, int *nlen, char **seq );
 extern int countATGC( char *s, int *total );
 extern void getnumlen( FILE *fp );
+extern void getnumlen_casepreserve( FILE *fp, int *nlenmin );
 extern void getnumlen_nogap( FILE *fp, int *nlenmin );
 extern void WriteGapFill( FILE *fp, int locnjob, char name[][B], int nlen[M], char **aseq );
 extern void writeDataforgaln( FILE *fp, int locnjob, char **name, int *nlen, char **aseq );
@@ -236,13 +239,15 @@ extern void outlocalhom( LocalHom **localhom, int nseq );
 extern void outlocalhompt( LocalHom ***localhom, int n1, int n2 );
 extern void FreeLocalHomTable( LocalHom **localhomtable, int n ) ;
 extern void constants( int nseq, char **seq );
-extern void clustalout_pointer( FILE *fp, int nseq, int maxlen, char **seq, char **name, char *mark, char *comment, int *order );
+extern void clustalout_pointer( FILE *fp, int nseq, int maxlen, char **seq, char **name, char *mark, char *comment, int *order, int namelen );
+extern void phylipout_pointer( FILE *fp, int nseq, int maxlen, char **seq, char **name, int *order );
 extern void writeData_reorder( FILE *fp, int locnjob, char name[][B], int nlen[], char **aseq, int *order );
 extern void writeData_reorder_pointer( FILE *fp, int locnjob, char **name, int *nlen, char **aseq, int *order );
 
 extern void resetlocalhom( int, LocalHom ** );
 extern int load1SeqWithoutName_new( FILE *fpp, char *cbuf );
 extern char *load1SeqWithoutName_realloc( FILE *fpp );
+extern char *load1SeqWithoutName_realloc_casepreserve( FILE *fpp );
 extern int disttbfast( char **in, int nlen[M], char name[M][B] );
 extern void searchKUorWA( FILE *fp );
 extern void gapireru( char *res, char *ori, char *gt );
