@@ -153,6 +153,32 @@ static void maskequiv( double *d, int n )
 }
 #endif
 
+static void makeequivdouble_tmalign( double *d, char *c, int n )
+{
+	double tmpd;
+	double *dbk;
+	int tmpi;
+	char s;
+	dbk = d;
+	while( *c )
+	{
+		if( ( s=*c++ ) == ':' )
+			tmpi = 9;
+		else if( s == '.' )
+			tmpi = 4;
+		else
+			tmpi = 0;
+//		tmpd = (double)( tmpi + 1 - equivthreshold ) / ( 10 - equivthreshold ) * 9.0;
+//		if( tmpd < 0.0 ) tmpd = 0.0;
+		tmpd = (double)( tmpi );
+//		*d++ = (int)tmpd;
+		*d++ = tmpd;
+	}
+
+	d = dbk;
+//	maskequiv( d, n );
+}
+
 static void makeequivdouble_threshold( double *d, char *c, int n )
 {
 	double tmpd;
@@ -172,6 +198,63 @@ static void makeequivdouble_threshold( double *d, char *c, int n )
 	maskequiv( d, n );
 }
 
+static void readtmalign( FILE *fp, char *seq1, char *seq2, double *equiv )
+{
+	static char *line = NULL;
+	static char *equivchar = NULL;
+	int n;
+
+	
+	if( equivchar == NULL )
+	{
+		equivchar = calloc( nlenmax * 2 + 1, sizeof( char ) );
+		line = calloc( nlenmax * 2 + 1, sizeof( char ) );
+	}
+	seq1[0] = 0;
+	seq2[0] = 0;
+	equivchar[0] = 0;
+
+
+//	system( "vi _tmalignout" );
+	while( 1 )
+	{
+		if( feof( fp ) ) 
+		{
+			fprintf( stderr, "Error in TMalign\n" );
+			exit( 1 );
+		}
+		fgets( line, 999, fp );
+//		fprintf( stdout, "line = :%s:\n", line );
+		if( !strncmp( line+5, "denotes the residue pairs", 20 ) ) break;
+	}
+	fgets( line, nlenmax*2, fp );
+	strcat( seq1, strip( line ) );
+
+	fgets( line, nlenmax*2, fp );
+	strcat( equivchar, strip( line ) );
+
+	fgets( line, nlenmax*2, fp );
+	strcat( seq2, strip( line ) );
+
+#if 0
+	printf( "seq1=%s\n", seq1 );
+	printf( "seq2=%s\n", seq2 );
+	printf( "equi=%s\n", equivchar );
+exit( 1 );
+#endif
+	n = strlen( seq1 );
+	makeequivdouble_tmalign( equiv, equivchar, n );
+
+#if 0
+	fprintf( stdout, "\n" );
+	for( i=0; i<n; i++ )
+	{
+		fprintf( stdout, "%1.0f", equiv[i] );
+	}
+	fprintf( stdout, "\n" );
+	exit( 1 );
+#endif
+}
 static void readrash( FILE *fp, char *seq1, char *seq2, double *equiv )
 {
 	char line[1000];
@@ -192,7 +275,7 @@ static void readrash( FILE *fp, char *seq1, char *seq2, double *equiv )
 		fgets( line, 999, fp );
 //		fprintf( stdout, "line = :%s:\n", line );
 		if( !strncmp( line, "Query ", 6 ) ) break;
-		if( feof( fp ) == 1 ) break;
+		if( feof( fp ) ) break;
 		if( !strncmp( line, "QUERY ", 6 ) )
 		{
 			strcat( seq1, strip( line+5 ) );
@@ -234,7 +317,7 @@ static int checkcbeta( FILE *fp )
 	while( 1 )
 	{
 		fgets( linec, 999, fp );
-		if( feof( fp ) == 1 ) break;
+		if( feof( fp ) ) break;
 		if( !strncmp( "ATOM ", linec, 5 ) )
 		{
 			if( !strncmp( "CB ", linec+13, 3 ) ) return( 0 );
@@ -244,20 +327,96 @@ static int checkcbeta( FILE *fp )
 }
 
 
-static float callrash( int mem1, int mem2, char **mseq1, char **mseq2, double *equiv, char *fname1, char *chain1, char *fname2, char *chain2, int alloclen )
+static float calltmalign( char **mseq1, char **mseq2, double *equiv, char *fname1, char *chain1, char *fname2, char *chain2, int alloclen )
 {
 	FILE *fp;
 	int res;
-	char com[10000];
-	char dirname[1000];
+	static char com[10000];
 	float value;
 	char cachedir[10000];
 	char cachefile[10000];
 	int runnow;
 
-	sprintf( dirname, "_rashtmp_%d-%d", mem1, mem2 );
-	sprintf( com, "rm -rf %s; mkdir %s", dirname, dirname );
-	system( com );
+
+	if( usecache )
+	{
+		sprintf( cachedir, "%s/.tmalignoutcache", getenv( "HOME" ) );
+		sprintf( com, "mkdir -p %s", cachedir );
+		system( com );
+
+		sprintf( cachefile, "%s/%s%s-%s%s", cachedir, fname1, chain1, fname2, chain2 );
+
+		runnow = 0;
+		fp = fopen( cachefile, "r" );
+		if( fp == NULL ) runnow = 1;
+		else
+		{
+			fgets( com, 100, fp );
+			if( strncmp( com, "successful", 10 ) ) runnow = 1;
+			fclose( fp );
+		}
+	}
+	else
+	{
+		runnow = 1;
+	}
+
+	if( runnow )
+	{
+#if 0
+		sprintf( com, "ln -s %s %s.pdb 2>_dum", fname1, fname1 );
+		res = system( com );
+		sprintf( com, "ln -s %s %s.pdb 2>_dum", fname2, fname2 );
+		res = system( com );
+#endif
+		sprintf( com, "\"%s/TMalign\"  %s.pdb %s.pdb > _tmalignout 2>_dum", whereispairalign, fname1, fname2 );
+		fprintf( stderr, "command = %s\n", com );
+		res = system( com );
+		if( res )
+		{
+			fprintf( stderr, "Error in TMalign\n" );
+			exit( 1 );
+		}
+	
+	}
+	else
+	{
+		fprintf( stderr, "Cache is not supported!\n" );
+		exit( 1 );
+	}
+
+	fp = fopen( "_tmalignout", "r" );
+	if( !fp )
+	{
+		fprintf( stderr, "Cannot open _tmalignout\n" );
+		exit( 1 );
+	}
+
+	readtmalign( fp, *mseq1, *mseq2, equiv );
+
+	fclose( fp );
+
+//	fprintf( stderr, "*mseq1 = %s\n", *mseq1 );
+//	fprintf( stderr, "*mseq2 = %s\n", *mseq2 );
+
+	value = (float)naivepairscore11( *mseq1, *mseq2, penalty );
+
+	return( value );
+}
+
+static float callrash( int mem1, int mem2, char **mseq1, char **mseq2, double *equiv, char *fname1, char *chain1, char *fname2, char *chain2, int alloclen )
+{
+	FILE *fp;
+	int res;
+	static char com[10000];
+	float value;
+	char cachedir[10000];
+	char cachefile[10000];
+	int runnow;
+	char pairid[1000];
+
+	sprintf( pairid, "%d-%d", mem1, mem2 );
+//	fprintf( stderr, "pairid = %s\n", pairid );
 
 	if( usecache )
 	{
@@ -290,24 +449,33 @@ static float callrash( int mem1, int mem2, char **mseq1, char **mseq2, double *e
 		sprintf( com, "ln -s %s %s.pdb 2>_dum", fname2, fname2 );
 		res = system( com );
 #endif
-#if 1  // 091127, pdp nai!
-		sprintf( com, "cd %s; %s/PDP_ASH.pl --qf ../%s.pdb --qc %s --tf ../%s.pdb --tc %s > _rashout 2>_dum", dirname, whereispairalign, fname1, chain1, fname2, chain2 );
+#if 0  // 091127, pdp nai!
+		sprintf( com, "env PATH=%s PDP_ASH.pl --qf %s.pdb --qc %s --tf %s.pdb --tc %s > _rashout 2>_dum", whereispairalign, fname1, chain1, fname2, chain2 );
 #else
-		sprintf( com, "cd %s; %s/rash --qf ../%s.pdb --qc %s --tf ../%s.pdb --tc %s > _rashout 2>_dum", dirname, whereispairalign, fname1, chain1, fname2, chain2 );
+		sprintf( com, "\"%s/rash\" --qf %s.pdb --qc %s --tf %s.pdb --tc %s --of %s.pdbpair > %s.rashout 2>%s.dum", whereispairalign, fname1, chain1, fname2, chain2, pairid, pairid, pairid );
 #endif
 		fprintf( stderr, "command = %s\n", com );
 		res = system( com );
 		if( res )
 		{
-			fprintf( stderr, "Error in PDP_ASH\n" );
+			fprintf( stderr, "Error in structural alignment\n" );
 			exit( 1 );
 		}
+		sprintf( com, "awk '/^REMARK/,/^TER/' %s.pdbpair > %s.%s-x-%s.%s.pdbpair", pairid, fname1, chain1, fname2, chain2 );
+		res = system( com );
+
+		sprintf( com, "awk '/^REMARK/,/^TER/{next} 1' %s.pdbpair > %s.%s-x-%s.%s.pdbpair", pairid, fname2, chain2, fname1, chain1 );
+		res = system( com );
+
+		sprintf( com, "rm %s.pdbpair", pairid );
+		res = system( com );
+
 	
 	}
 	else
 	{
 		fprintf( stderr, "Use cache!\n" );
-		sprintf( com, "grep -v successful %s > %s/_rashout", dirname, cachefile );
+		sprintf( com, "grep -v successful %s > %s.rashout", cachefile, pairid );
 		system( com );
 	}
 
@@ -315,25 +483,22 @@ static float callrash( int mem1, int mem2, char **mseq1, char **mseq2, double *e
 	{
 		sprintf( com, "echo successful >  %s", cachefile );
 		system( com );
-		sprintf( com, "cat _rashout >>  %s", cachefile );
+		sprintf( com, "cat %s.rashout >>  %s", pairid, cachefile );
 		system( com );
 	}
 
-	sprintf( com, "%s/_rashout", dirname );
+	sprintf( com, "%s.rashout", pairid );
 	fp = fopen( com, "r" );
 	if( !fp )
 	{
-		fprintf( stderr, "Cannot open _rashout\n" );
+		fprintf( stderr, "Cannot open %s\n", com );
 		exit( 1 );
 	}
 
 	readrash( fp, *mseq1, *mseq2, equiv );
 
-
 	fclose( fp );
 
-	sprintf( com, "rm -rf %s", dirname );
-
 //	fprintf( stderr, "*mseq1 = %s\n", *mseq1 );
 //	fprintf( stderr, "*mseq2 = %s\n", *mseq2 );
 
@@ -343,9 +508,9 @@ static float callrash( int mem1, int mem2, char **mseq1, char **mseq2, double *e
 	return( value );
 }
 
-static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***seqpt, char ***mseq1pt, char ***mseq2pt, double **equivpt, int *alloclenpt )
+static void preparetmalign( FILE *fp, char ***strfiles, char ***chainids, char ***seqpt, char ***mseq1pt, char ***mseq2pt, double **equivpt, int *alloclenpt )
 {
-	int i;
+	int i, res;
 	char *dumseq;
 	char line[1000];
 	char fname[1000];
@@ -359,7 +524,7 @@ static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***se
 	while( 1 )
 	{
 		fgets( line, 999, fp );
-		if( feof( fp ) == 1 ) break;
+		if( feof( fp ) ) break;
 		sline = strip( line );
 		use[linenum] = 1;
 		if( sline[0] == '#' || strlen( sline ) < 2 )
@@ -405,7 +570,7 @@ static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***se
 	while( 1 )
 	{
 		fgets( line, 999, fp );
-		if( feof( fp ) == 1 ) break;
+		if( feof( fp ) ) break;
 		sline = strip( line );
 		if( use[linenum++] ) 
 		{
@@ -416,7 +581,13 @@ static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***se
 			cutpath( fname );
 			sprintf( command, "cp %s %s.pdb", sline, fname );
 			system( command );
-//			fprintf( stderr, "command = %s\n", command );
+			sprintf( command, "perl \"%s/clean.pl\" %s.pdb", whereispairalign, fname );
+			res = system( command );
+			if( res )
+			{
+				fprintf( stderr, "error: Install clean.pl\n" );
+				exit( 1 );
+			}
 			strcpy( (*strfiles)[istr++], fname );
 		}
 	}
@@ -430,29 +601,131 @@ static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***se
 	alreadyoutput = AllocateIntVec( njob );
 	for( i=0; i<njob; i++ ) alreadyoutput[i] = 0;
 
-#if 1
+	for( i=0; i<istr; i++ )
 	{
-		for( i=0; i<istr; i++ )
+		fprintf( stderr, "i=%d\n", i );
+		(*seqpt)[i][0] = 0;
+
+		(*mseq1pt)[0] = (*seqpt)[i];
+		(*mseq2pt)[0] = dumseq;
+
+		calltmalign( *mseq1pt, *mseq2pt, *equivpt, (*strfiles)[i], (*chainids)[i], (*strfiles)[i], (*chainids)[i], *alloclenpt );
+		fprintf( stdout, ">%d_%s-%s\n%s\n", i+1, (*strfiles)[i], (*chainids)[i], (*seqpt)[i] );
+		alreadyoutput[i] = 1;
+	}
+}
+
+static void prepareash( FILE *fp, char ***strfiles, char ***chainids, char ***seqpt, char ***mseq1pt, char ***mseq2pt, double **equivpt, int *alloclenpt )
+{
+	int i, res;
+	char *dumseq;
+	char line[1000];
+	char fname[1000];
+	char command[1000];
+	int linenum, istr, nstr;
+	FILE *checkfp;
+	char *sline; 
+	int use[1000];
+	linenum = 0;
+	nstr = 0;
+	while( 1 )
+	{
+		fgets( line, 999, fp );
+		if( feof( fp ) ) break;
+		sline = strip( line );
+		use[linenum] = 1;
+		if( sline[0] == '#' || strlen( sline ) < 2 )
 		{
-			fprintf( stderr, "i=%d\n", i );
-			(*seqpt)[i][0] = 0;
-	
-			(*mseq1pt)[0] = (*seqpt)[i];
-			(*mseq2pt)[0] = dumseq;
-	
-			callrash( i, i, *mseq1pt, *mseq2pt, *equivpt, (*strfiles)[i], (*chainids)[i], (*strfiles)[i], (*chainids)[i], *alloclenpt );
-			fprintf( stdout, ">%d_%s-%s\n%s\n", i+1, (*strfiles)[i], (*chainids)[i], (*seqpt)[i] );
-			alreadyoutput[i] = 1;
+			use[linenum] = 0;
+			linenum++;
+			continue;
+		}
+		extractfirstword( sline );
+		checkfp = fopen( sline, "r" );
+		if( checkfp == NULL )
+		{
+			fprintf( stderr, "Cannot open %s.\n", sline );
+			exit( 1 );
+		}
+#if 0
+		fgets( linec, 999, checkfp );
+		if( strncmp( "HEADER ", linec, 7 ) )
+		{
+			fprintf( stderr, "Check the format of %s.\n", sline );
+			exit( 1 );
+		}
+#endif
+		if( checkcbeta( checkfp ) ) 
+		{
+			fprintf( stderr, "%s has no C-beta atoms.\n", sline );
+			exit( 1 );
+		}
+		else
+			nstr++;
+		fclose( checkfp );
+		linenum++;
+	}
+	njob = nstr;
+	fprintf( stderr, "nstr = %d\n", nstr );
+
+	*strfiles = AllocateCharMtx( nstr, 1000 );
+	*chainids = AllocateCharMtx( nstr, 2 );
+
+	rewind( fp );
+	istr = 0;
+	linenum = 0;
+	while( 1 )
+	{
+		fgets( line, 999, fp );
+		if( feof( fp ) ) break;
+		sline = strip( line );
+		if( use[linenum++] ) 
+		{
+			(*chainids)[istr][0] = getchainid( sline );
+			(*chainids)[istr][1] = 0;
+			extractfirstword( sline );
+			sprintf( fname, "%s", sline );
+			cutpath( fname );
+			sprintf( command, "cp %s %s.pdb", sline, fname );
+			system( command );
+			sprintf( command, "perl \"%s/clean.pl\" %s.pdb", whereispairalign, fname );
+			res = system( command );
+			if( res )
+			{
+				fprintf( stderr, "error: Install clean.pl\n" );
+				exit( 1 );
+			}
+			strcpy( (*strfiles)[istr++], fname );
 		}
 	}
-#endif
+
+	*seqpt = AllocateCharMtx( njob, nlenmax*2+1 );
+	*mseq1pt = AllocateCharMtx( njob, 0 );
+	*mseq2pt = AllocateCharMtx( njob, 0 );
+	*equivpt = AllocateDoubleVec( nlenmax*2+1 );
+	*alloclenpt = nlenmax*2;
+	dumseq = AllocateCharVec( nlenmax*2+1 );
+	alreadyoutput = AllocateIntVec( njob );
+	for( i=0; i<njob; i++ ) alreadyoutput[i] = 0;
+
+	for( i=0; i<istr; i++ )
+	{
+		fprintf( stderr, "i=%d\n", i );
+		(*seqpt)[i][0] = 0;
+
+		(*mseq1pt)[0] = (*seqpt)[i];
+		(*mseq2pt)[0] = dumseq;
+
+		callrash( i, i, *mseq1pt, *mseq2pt, *equivpt, (*strfiles)[i], (*chainids)[i], (*strfiles)[i], (*chainids)[i], *alloclenpt );
+		fprintf( stdout, ">%d_%s-%s\n%s\n", i+1, (*strfiles)[i], (*chainids)[i], (*seqpt)[i] );
+		alreadyoutput[i] = 1;
+	}
 }
 
 void arguments( int argc, char *argv[] )
 {
     int c;
 
-	nthread = 0;
 	usecache = 0;
 	scale = 1.0;
 	equivthreshold = 5;
@@ -581,10 +854,6 @@ void arguments( int argc, char *argv[] )
 					scale = atof( *++argv );
 					--argc;
 					goto nextoption;
-				case 'C':
-					nthread = atoi( *++argv );
-					--argc;
-					goto nextoption;
 				case 'c':
 					usecache = 1;
 					break;
@@ -605,10 +874,85 @@ void arguments( int argc, char *argv[] )
 				case 'e':
 					fftscore = 0;
 					break;
+#if 0
+				case 'O':
+					fftNoAnchStop = 1;
+					break;
+#endif
+				case 'Q':
+					calledByXced = 1;
+					break;
+				case 'x':
+					disp = 1;
+					break;
+#if 0
+				case 'a':
+					alg = 'a';
+					break;
+#endif
+				case 'S':
+					alg = 'S';
+					break;
+				case 'L':
+					alg = 'L';
+					break;
+				case 'B':
+					alg = 'B';
+					break;
+				case 'T':
+					alg = 'T';
+					break;
+				case 'H':
+					alg = 'H';
+					break;
+				case 'M':
+					alg = 'M';
 					break;
 				case 'R':
 					alg = 'R';
 					break;
+				case 'N':
+					alg = 'N';
+					break;
+				case 'K':
+					alg = 'K';
+					break;
+				case 'A':
+					alg = 'A';
+					break;
+				case 'V':
+					alg = 'V';
+					break;
+				case 'C':
+					alg = 'C';
+					break;
+				case 'F':
+					use_fft = 1;
+					break;
+				case 'v':
+					tbrweight = 3;
+					break;
+				case 'y':
+					divpairscore = 1;
+					break;
+/* Modified 01/08/27, default: user tree */
+				case 'J':
+					tbutree = 0;
+					break;
+/* modification end. */
+#if 0
+				case 'z':
+					fftThreshold = atoi( *++argv );
+					--argc; 
+					goto nextoption;
+				case 'w':
+					fftWinSize = atoi( *++argv );
+					--argc;
+					goto nextoption;
+				case 'Z':
+					checkC = 1;
+					break;
+#endif
                 default:
                     fprintf( stderr, "illegal option %c\n", c );
                     argc = 0;
@@ -726,17 +1070,52 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 	{
 		fprintf( stderr, "% 5d / %d\r", i, njob );
 
+
 		for( j=i+1; j<njob; j++ )
 		{
+
+
+#if 0
+			if( strlen( seq[i] ) == 0 || strlen( seq[j] ) == 0 )
+			{
+				distancemtx[i][j] = pscore;
+				continue;
+			}
+#endif
+
 			strcpy( aseq[i], seq[i] );
 			strcpy( aseq[j], seq[j] );
 			clus1 = conjuctionfortbfast( pair, i, aseq, mseq1, effarr1, effarr, indication1 );
 			clus2 = conjuctionfortbfast( pair, j, aseq, mseq2, effarr2, effarr, indication2 );
+	//		fprintf( stderr, "mseq1 = %s\n", mseq1[0] );
+	//		fprintf( stderr, "mseq2 = %s\n", mseq2[0] );
 	
-			fprintf( stderr, "  Calling PDP_ASH.pl %d-%d/%d    \r", i+1, j+1, njob );
-			pscore = callrash( i, j, mseq1, mseq2, equiv, strfiles[i], chainids[i], strfiles[j], chainids[j], alloclen );
-			off1 = off2 = 0;
-			break;
+#if 0
+			fprintf( stderr, "group1 = %.66s", indication1 );
+			fprintf( stderr, "\n" );
+			fprintf( stderr, "group2 = %.66s", indication2 );
+			fprintf( stderr, "\n" );
+#endif
+//			for( l=0; l<clus1; l++ ) fprintf( stderr, "## STEP-eff for mseq1-%d %f\n", l, effarr1[l] );
+	
+#if 1
+			{
+				switch( alg )
+				{
+					case( 'T' ):
+						fprintf( stderr, "  Calling tmalign %d-%d/%d    \r", i+1, j+1, njob );
+						pscore = calltmalign( mseq1, mseq2, equiv, strfiles[i], chainids[i], strfiles[j], chainids[j], alloclen );
+						off1 = off2 = 0;
+						break;
+					case( 'R' ):
+						fprintf( stderr, "  Calling PDP_ASH.pl %d-%d/%d    \r", i+1, j+1, njob );
+						pscore = callrash( i, j, mseq1, mseq2, equiv, strfiles[i], chainids[i], strfiles[j], chainids[j], alloclen );
+						off1 = off2 = 0;
+						break;
+					ErrorExit( "ERROR IN SOURCE FILE" );
+				}
+			}
+#endif
 			distancemtx[i][j] = pscore;
 #if SCOREOUT
 			fprintf( stderr, "score = %10.2f (%d,%d)\n", pscore, i, j );
@@ -766,13 +1145,14 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 					fprintf( stderr, "i=%d\n", i );
 					fprintf( stderr, "     seq=%s\n", seq[i] );
 					fprintf( stderr, "checkseq=%s\n", checkseq[i] );
-//					exit( 1 );
+					exit( 1 );
 				}
 			}
 			if( alreadyoutput[j] == 0 )
 			{
 				alreadyoutput[j] = 1;
 				gappick0( seq[j], mseq2[0] );
+				fprintf( stdout, ">%d_%s-%s\n%s\n", j+1, strfiles[j], chainids[j], seq[j] );
 				strcpy( checkseq[j], seq[j] );
 			}
 			else
@@ -785,16 +1165,12 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 					fprintf( stderr, "j=%d\n", j );
 					fprintf( stderr, "     seq=%s\n", seq[j] );
 					fprintf( stderr, "checkseq=%s\n", checkseq[j] );
-//					exit( 1 );
+					exit( 1 );
 				}
 			}
 #endif
 		}
 	}
-
-//	for( i=0; i<njob; i++ )
-//		fprintf( stdout, ">%d_%s-%s\n%s\n", i+1, strfiles[i], chainids[i], checkseq[i] );
-
 	for( i=0; i<njob; i++ )
 	{
 		pscore = 0.0;
@@ -833,11 +1209,7 @@ static void pairalign( char name[M][B], int nlen[M], char **seq, char **aseq, ch
 			for( tmpptr=localhomtable[i]+j; tmpptr; tmpptr=tmpptr->next )
 			{
 				if( tmpptr->opt == -1.0 ) continue;
-// tmptmptmptmptmp
-//				if( alg == 'B' || alg == 'T' )
-//					fprintf( hat3p, "%d %d %d %7.5f %d %d %d %d %p\n", i, j, tmpptr->overlapaa, 1.0, tmpptr->start1, tmpptr->end1, tmpptr->start2, tmpptr->end2, (void *)tmpptr->next ); 
-//				else
-					fprintf( hat3p, "%d %d %d %7.5f %d %d %d %d k\n", i, j, tmpptr->overlapaa, tmpptr->opt, tmpptr->start1, tmpptr->end1, tmpptr->start2, tmpptr->end2 ); 
+				fprintf( hat3p, "%d %d %d %7.5f %d %d %d %d k\n", i, j, tmpptr->overlapaa, tmpptr->opt, tmpptr->start1, tmpptr->end1, tmpptr->start2, tmpptr->end2 ); 
 			}
 		}
 	}
@@ -927,9 +1299,6 @@ int main( int argc, char *argv[] )
 	int alloclen;
 
 	arguments( argc, argv );
-#ifndef enablemultithread
-	nthread = 0;
-#endif
 
 	if( equivthreshold < 1 || 9 < equivthreshold )
 	{
@@ -958,7 +1327,11 @@ int main( int argc, char *argv[] )
 
 	nlenmax = 10000; // tekitou
 
-	prepareash( infp, &strfiles, &chainids, &seq, &mseq1, &mseq2, &equiv, &alloclen );
+	if( alg == 'R' )
+		prepareash( infp, &strfiles, &chainids, &seq, &mseq1, &mseq2, &equiv, &alloclen );
+	else if( alg == 'T' )
+		preparetmalign( infp, &strfiles, &chainids, &seq, &mseq1, &mseq2, &equiv, &alloclen );
+
 	fclose( infp );
 
 	aseq = AllocateCharMtx( njob, nlenmax*2+1 );
