@@ -161,7 +161,7 @@ static void calllara( int nseq, char **mseq, char *laraarg )
 	int res;
 	static char com[10000];
 
-	for( i=0; i<nseq; i++ )
+//	for( i=0; i<nseq; i++ )
 
 	fp = fopen( "_larain", "w" );
 	if( !fp )
@@ -1004,11 +1004,11 @@ static void pairalign( char **name, int nlen[M], char **seq, char **aseq, char *
 	double *effarr2;
 	char *pt;
 	char *hat2file = "hat2";
-	LocalHom **localhomtable, *tmpptr;
+	LocalHom **localhomtable = NULL, *tmpptr;
 	static char **pair;
 	int intdum;
 	double bunbo;
-	char ***bpp; // mxscarna no toki dake
+	char ***bpp = NULL; // mxscarna no toki dake
 
 	if( store_localhom )
 	{
