@@ -107,6 +107,13 @@ void part_imp_match_init_strict( float *imp, int clus1, int clus2, int lgth1, in
 	char *pt, *pt1, *pt2;
 	LocalHom *tmpptr;
 
+	if( seq1 == NULL )
+	{
+		if( impmtx ) FreeFloatMtx( impmtx );
+		impmtx = NULL;
+		return;
+	}
+
 	if( impalloclen <= lgth1 + 2 || impalloclen <= lgth2 + 2 )
 	{
 		if( impmtx ) FreeFloatMtx( impmtx );
@@ -734,7 +741,7 @@ static float Atracking( float *lasthorizontalw, float *lastverticalw,
 	return( 0.0 );
 }
 
-float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc, int jcyc, int alloclen, LocalHom ***localhom, float *impmatch, int start1, int end1, int start2, int end2, int *gapmap1, int *gapmap2, char *sgap1, char *sgap2, char *egap1, char *egap2 )
+float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int icyc, int jcyc, int alloclen, LocalHom ***localhom, float *impmatch, int start1, int end1, int start2, int end2, int *gapmap1, int *gapmap2, char *sgap1, char *sgap2, char *egap1, char *egap2, int *chudanpt, int chudanref, int *chudanres )
 /* score no keisan no sai motokaraaru gap no atukai ni mondai ga aru */
 {
 //	int k;
@@ -780,6 +787,49 @@ float partA__align( char **seq1, char **seq2, double *eff1, double *eff2, int ic
 	float ogcp1va;
 
 
+	if( seq1 == NULL )
+	{
+		if( orlgth1 )
+		{
+//			fprintf( stderr, "## Freeing local arrays in A__align\n" );
+			orlgth1 = 0;
+			orlgth2 = 0;
+
+			part_imp_match_init_strict( NULL, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0 );
+
+			free( mseq1 );
+			free( mseq2 );
+			FreeFloatVec( w1 );
+			FreeFloatVec( w2 );
+			FreeFloatVec( match );
+			FreeFloatVec( initverticalw );
+			FreeFloatVec( lastverticalw );
+
+			FreeFloatVec( m );
+			FreeIntVec( mp );
+
+			FreeCharMtx( mseq );
+
+			FreeFloatVec( ogcp1 );
+			FreeFloatVec( ogcp2 );
+			FreeFloatVec( fgcp1 );
+			FreeFloatVec( fgcp2 );
+
+
+			FreeFloatMtx( cpmx1 );
+			FreeFloatMtx( cpmx2 );
+
+			FreeFloatMtx( floatwork );
+			FreeIntMtx( intwork );
+
+		}
+		else
+		{
+//			fprintf( stderr, "## Not allocated\n" );
+		}
+		return( 0.0 );
+	}
+
 
 #if 0
 	fprintf( stderr, "eff in SA+++align\n" );
@@ -1004,6 +1054,17 @@ for( i=0; i<lgth2; i++ )
 
 	for( i=1; i<lasti; i++ )
 	{
+
+#ifdef enablemultithread
+//		fprintf( stderr, "chudan = %d, %d\n", *chudanpt, chudanref );
+		if( chudanpt && *chudanpt != chudanref ) 
+		{
+//			fprintf( stderr, "\n\n## CHUUDAN!!! i\n" );
+			*chudanres = 1;
+			return( -1.0 );
+		}
+#endif
+
 		wtmp = previousw; 
 		previousw = currentw;
 		currentw = wtmp;
@@ -1053,6 +1114,15 @@ fprintf( stderr, "\n" );
 		ogcp1va = ogcp1[i];
 		for( j=1; j<lastj; j++ )
 		{
+#ifdef xxxenablemultithread
+//			fprintf( stderr, "chudan = %d, %d\n", *chudanpt, chudanref );
+			if( chudanpt && *chudanpt != chudanref ) 
+			{
+//				fprintf( stderr, "\n\n## CHUUDAN!!! j\n" );
+				*chudanres = 1;
+				return( -1.0 );
+			}
+#endif
 			wm = *prept;
 			*ijppt = 0;
 
@@ -1110,6 +1180,7 @@ fprintf( stderr, "\n" );
 			ogcp2pt++;
 		}
 		lastverticalw[i] = currentw[lgth2-1];
+
 	}
 
 #if OUTGAP0TRY
