@@ -216,7 +216,7 @@ void rnaalifoldcall( char **seq, int nseq, RNApair **pairprob )
 		fprintf( stderr, "Cannot open /tmp/_rnaalifoldin\n" );
 		exit( 1 );
 	}
-	clustalout_pointer( fp, nseq, lgth, seq, name, NULL, NULL, order );
+	clustalout_pointer( fp, nseq, lgth, seq, name, NULL, NULL, order, 15 );
 	fclose( fp );
 
 	sprintf( cmd, "RNAalifold -p %s", fnamein );
